package beasts;

import java.awt.*;

/**
 * Abstract base class of beasts.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@unsw.edu.au">peteg@unsw.edu.au</A>)
 * @version $Id: Beast.java,v 1.8 2002/02/03 23:47:16 peteg Exp $
 */
abstract class Beast extends WorldObject implements Event {
    /**
     * Flags a beast as dead; saves hunting through the queue to delete it.
     */
    boolean dead;

    Beast(BeastsWorld beasts, int x, int y) {
	super(beasts, x, y);
	dead = false;
    }

    /**
     * When this beast is killed, how many points does the player get?
     */
    abstract int points();

    boolean push(boolean player, int dx, int dy) {
	WorldObject wo = beasts.getWO(x + dx, y + dy);

	/* Check to see if we got squashed, otherwise disallow the move. */
	if(wo instanceof Block) {
	    beasts.removeWO(this);
	    died();

	    /* Player only gets the points if they pushed the block. */
	    if(player) {
		beasts.addScore(points());
	    }

	    return true;
	} else {
	    return false;
	}
    }

    void died() {
	dead = true;
    }

    /**
     * Most beasts can ignore the signal.
     */ 
    public void signal() {
    }
}
