package beasts;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.zip.*;

/**
 * Beasts Applet and Application.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@unsw.edu.au">peteg@unsw.edu.au</A>)
 * @version $Id: Beasts.java,v 1.15 2002/03/20 08:16:24 peteg Exp $
 */
public class Beasts extends Applet {
    /**
     * GUI Wrapper so the Beasts applet can be run standalone.
     */
    private static class BeastsWindow extends Frame {
	BeastsWorld beasts;

	BeastsWindow(PersistentState ps, boolean training) {
	    super("Beasts");

	    this.setIconImage(Toolkit.getDefaultToolkit().getImage(Beasts.class.getResource("BeastsIcon.gif")));

	    this.add(beasts = new BeastsWorld(ps, training));
	    this.setResizable(false);
	    this.pack();

	    this.addWindowListener(new WindowAdapter() {
		    public void windowClosing(WindowEvent e) {
			beasts.quit(BeastsWorld.QUIT_GAME);
		    }
		});
	}
    }

    private BeastsWorld beasts;

    public Insets getInsets() {
	return new Insets(0, 0, 0, 0);
    }

    public Dimension getMinimumSize() {
	return getPreferredSize();
    }

    public Dimension getPreferredSize() {
	return new Dimension(BeastsWorld.WIDTH, BeastsWorld.HEIGHT);
    }

    public String getAppletInfo() {
	return("Beasts Applet - Peter Gammie, 2002.");
    }

    public String[][] getParameterInfo() {
	String info[][] = {
	};

	return(info);
    }

    public void init() {
	this.add(beasts = new BeastsWorld(new PersistentState(false), false));
    }

    public void start() {
	showStatus("Beasts running.");
	new Thread(beasts).start();
    }

    public void destroy() {
    }

    public void stop() {
    }

    public void paint(Graphics g) {
	update(g);
    }

    public void update(Graphics g) {
    }

    /**
     * Standalone entry point.
     */
    public static void main(String[] argv) {
	boolean training = false;
	PersistentState ps = null; // System.exit() not handled by control flow analysis.

	/* Check for command-line arguments. */
	if(argv.length > 0) {
	    if(argv[0].equals("-t")) {
		training = true;
	    } else {
		System.err.println("\nUsage: beasts.Beasts [-t]\n\n" +
				   "  -t\tTraining mode (infinite player lives).\n");
		    System.exit(1);
	    }
	}

	/* Load the high score table and beasts learning state. */
	String fname = System.getProperty("beasts.stateFile");

	try {
	    InputStream file = (fname == null)
		? System.in
		: new FileInputStream(fname);
	    ObjectInputStream in = new ObjectInputStream(new GZIPInputStream(file));
	    ps = (PersistentState)in.readObject();
	    in.close();
	} catch (FileNotFoundException e) {
	    /* Create a new one. */
	    System.err.println("beasts: persistent state file: \"" + fname + "\" not found; will be created on exit.");
	    ps = new PersistentState(true);
	} catch (IOException e) {
	    /* Create a new one. */
	    System.err.println("beasts: persistent state file: \"" + fname + "\" not found; will be created on exit.");
	    ps = new PersistentState(true);
	} catch (ClassNotFoundException e) {
	    System.err.println("beasts: error loading \"beasts.stateFile\": \"" + fname + "\": " + e.toString());
	    System.exit(1);
	}

	BeastsWindow beastsWindow = new BeastsWindow(ps, training);
        beastsWindow.setVisible(true);

	/* No need to start a thread; it can use this one. */
	beastsWindow.beasts.run();

	/* Save the high score table and beasts learning state. */
	try {
	    OutputStream file = (fname == null)
		? (OutputStream)System.out
		: new FileOutputStream(fname);
	    ObjectOutputStream out = new ObjectOutputStream(new GZIPOutputStream(file));
	    out.writeObject(ps);
	    out.close();
	} catch (IOException e) {
	    System.err.println("beasts: error saving \"beasts.stateFile\": \"" + fname + "\": " + e.toString());
	    System.exit(1);
	}

	System.exit(0);
    }
}
