package beasts;

/**
 * Represents the characteristics of the levels in <code>Beasts</code>. Uses
 * the singleton pattern.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@unsw.edu.au">peteg@unsw.edu.au</A>)
 * @version $Id: Levels.java,v 1.9 2002/03/15 02:39:40 peteg Exp $
 */
class Levels {
    private static class Level {
	boolean pullBlocks;
	boolean gameSpeedUp;
	boolean explosiveBlocks;
	int superBeastRatio;
	int eggs;
	int winningRound;

	int hatchingSpeed;
	int startingBeasts;

	Level(boolean pullBlocks,
	      boolean gameSpeedUp,
	      boolean explosiveBlocks,
	      int superBeastRatio,
	      int eggs,
	      int winningRound,
	      int hatchingSpeed,
	      int startingBeasts) {
	    this.pullBlocks = pullBlocks;
	    this.gameSpeedUp = gameSpeedUp;
	    this.explosiveBlocks = explosiveBlocks;
	    this.superBeastRatio = superBeastRatio;
	    this.eggs = eggs;
	    this.winningRound = winningRound;

	    this.hatchingSpeed = hatchingSpeed;
	    this.startingBeasts = startingBeasts;
	}

	void displayLevel(BeastsWorld beasts) {
	    StringFormatter out = new StringFormatter();

	    beasts.drawString("Pull Blocks         : " + (pullBlocks ? "yes" : "no"), 10, 4);
	    beasts.drawString("Game Speed Up       : " + (gameSpeedUp ? "yes" : "no"), 10, 5);
	    beasts.drawString("Explosive Blocks    : " + (explosiveBlocks ? "yes" : "no"), 10, 6);

	    out.append("Super Beast ratio   : ");
	    switch(superBeastRatio) {
	    case 0:
		out.append("none");
		break;

	    case 1:
		out.append("all");
		break;

	    default:
		out.append("1 out of ");
		out.append(superBeastRatio);
		break;
	    }
	    beasts.drawString(out.toString(), 10, 7);
	    out.clear();

	    out.append("Eggs                : ");
	    if(eggs > 0) {
		out.append(eggs, 2, StringFormatter.RIGHT, ' ');
	    } else {
		out.append("none");
	    }
	    beasts.drawString(out.toString(), 10, 8);
	    out.clear();

	    out.append("Winning Round       : ");
	    out.append(winningRound, 2, StringFormatter.RIGHT, ' ');
	    beasts.drawString(out.toString(), 10, 9);
	    out.clear();

	    out.append("Hatching Speed      : ");
	    out.append(hatchingSpeed, 2, StringFormatter.RIGHT, ' ');
	    beasts.drawString(out.toString(), 10, 11);
	    out.clear();

	    out.append("Starting Beasts     : ");
	    out.append(startingBeasts, 2, StringFormatter.RIGHT, ' ');
	    beasts.drawString(out.toString(), 10, 12);
	    out.clear();
	}

	void initLevel(BeastsWorld beasts) {
	    beasts.initLevel(pullBlocks,
			     winningRound);
	    beasts.initRound(gameSpeedUp,
			     explosiveBlocks,
			     superBeastRatio,
			     eggs,
			     hatchingSpeed,
			     startingBeasts);
	}

	/**
	 * Initialise the next round of this level.
	 * <ul>
	 *  <li> Increase the number of beasts and eggs by 10%.
	 * </ul>
	 */
	void nextRound(BeastsWorld beasts, int round) {
	    int increaseBy = (int)(Math.pow(1.1, round) + 0.5);

	    beasts.initRound(gameSpeedUp,
			     explosiveBlocks,
			     superBeastRatio,
			     eggs * increaseBy,
			     hatchingSpeed,
			     startingBeasts * increaseBy);
	}
    }

    private static Level[] levels = {
	new Level(true,  false, false, 0,  0,  2,  0,  1), // A
	new Level(true,  false, false, 0,  0,  2,  0,  2), // B
	new Level(true,  false, false, 0,  0,  2,  0,  3), // C
	new Level(true,  false, false, 0,  0,  3,  0,  3), // D
	new Level(true,  false, false, 0,  0,  3,  0,  4), // E
	new Level(true,  true,  false, 0,  0,  3,  0,  4), // F
	new Level(true,  true,  false, 0,  1,  3, 60,  4), // G
	new Level(true,  true,  false, 8,  0,  6,  0,  4), // H
	new Level(true,  true,  false, 8,  1,  6, 60,  4), // I
	new Level(false, true,  false, 8,  2,  6, 60,  4), // J
	new Level(false, true,  false, 8,  3,  6, 60,  4), // K

	new Level(true,  true,  true,  1, 20,  6,  0, 20), // L user
	new Level(true,  true,  true,  8,  1,  6,  0, 10), // M user
	new Level(true,  false, true,  8, 15,  6,  7,  1), // N user
	new Level(true,  false, true,  8,  1,  6, 50,  4), // O user

	new Level(false, true,  true,  8,  5,  9, 50,  4), // P
	new Level(false, true,  true,  4,  6,  9, 50,  4), // Q
	new Level(false, true,  true,  4,  7,  9, 50,  4), // R
	new Level(false, true,  true,  4,  8,  9, 50,  4), // S
	new Level(false, true,  true,  8,  8, 15, 45,  4), // T
	new Level(false, true,  true,  8,  9, 15, 40,  5), // U
	new Level(false, true,  true,  4, 10, 15, 35,  5), // V
	new Level(false, true,  true,  4, 10, 15, 30,  6), // W
	new Level(false, true,  true,  4, 10, 20, 20,  6), // Y
	new Level(false, true,  true,  2, 10, 20, 15,  6), // Y
	new Level(false, true,  true,  1, 10, 20, 15,  2), // Z
    };

    private Levels() {
    }

    static void displayLevel(BeastsWorld beasts, int level) {
	beasts.clear();

	/* Add a border of immovable block images. */
	for(int i = 0; i < BeastsWorld.COLS; i++) {
	    beasts.draw(i, 0,                    HeavyBlock.image);
	    beasts.draw(i, BeastsWorld.ROWS - 1, HeavyBlock.image);
	}

	for(int i = 1; i < BeastsWorld.ROWS - 1; i++) {
	    beasts.draw(0,                    i, HeavyBlock.image);
	    beasts.draw(BeastsWorld.COLS - 1, i, HeavyBlock.image);
	}

	/* Add the header. */
	beasts.drawString("** Current Options Installed **", 15, 1);

	levels[level].displayLevel(beasts);
	beasts.updateStatus(true);
    }

    static void initLevel(BeastsWorld beasts, int level) {
	levels[level].initLevel(beasts);
//  	beasts.initLevel(true, 10);
//  	beasts.initRound(false, false, 0, 0, 0, 1);
    }

    static void nextRound(BeastsWorld beasts, int level, int round) {
	levels[level].nextRound(beasts, round);
    }

    static String levelToString(int level, int round) {
	StringBuffer out = new StringBuffer();

	out.append(round);
	out.append((char)(level + 'A'));
	out.append(" ");

	if(level <= ('C' - 'A')) {
	    out.append("EASY");
	} else if(level <= ('G' - 'A')) {
	    out.append("NOVICE");
	} else if(level <= ('K' - 'A')) {
	    out.append("HARD");
	} else if(level <= ('O' - 'A')) {
	    out.append("UNKNOWN");
	} else if(level <= ('S' - 'A')) {
	    out.append("ADVANCED");
	} else if(level <= ('W' - 'A')) {
	    out.append("EXPERT");
	} else {
	    out.append("PRO");
	}

	return out.toString();
    }
}
