package beasts;

import java.io.*;

/**
 * Aggregates all the persistent state of the game, to make serialisation
 * easy. Also provides factory methods for objects that depend on whether
 * we're running as an application or an applet.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@unsw.edu.au">peteg@unsw.edu.au</A>)
 * @version $Id: PersistentState.java,v 1.4 2002/03/15 07:47:04 peteg Exp $
 */
public class PersistentState implements Serializable {
    private Scores scores;
    private RLearner bbrl;
    private boolean application;

    public PersistentState(boolean application) {
	this.application = application;
	this.scores = new Scores();

	if(application) {
	    bbrl = RLBasicBeast.newRL();
	}
    }

    Scores getScores() {
	return scores;
    }

    BasicBeast newBasicBeast(BeastsWorld beasts, int x, int y) {
// 	return new BasicBeast(beasts, x, y);
	return application
	    ? new RLBasicBeast(beasts, x, y, bbrl)
	    : new BasicBeast(beasts, x, y);
    }
}
