package beasts;

import java.awt.*;
import java.awt.event.*;

/**
 * The player.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@unsw.edu.au">peteg@unsw.edu.au</A>)
 * @version $Id: Player.java,v 1.15 2002/03/15 07:47:04 peteg Exp $
 */
class Player extends WorldObject implements KeyListener {
    static Image image;
    private int lives;

    /**
     * Can we pull blocks?
     */
    private boolean pull;

    private final static int LIVES = 5;

    Player(BeastsWorld beasts) {
	super(beasts);
    }

    /**
     * Called at the start of each level.
     */
    void initLevel(boolean pull) {
	this.lives = LIVES;
	this.pull = pull;
    }

    /**
     * Called at the start of each round.
     */
    void initRound() {
	beasts.addPlayer();
	beasts.addKeyListener(this);
    }

    void finishedRound() {
	beasts.removeKeyListener(this);
    }

    /**
     * Handles a key press from the user.
     */
    public void keyPressed(KeyEvent ke) {
	int dx, dy;

// 	System.err.println("Player.keyPressed(): " + Thread.currentThread().toString());
//   	System.err.println("Player.keyPressed(): " + ke);

	switch(ke.getKeyCode()) {
	    /* Movement keys. */
	case KeyEvent.VK_NUMPAD7:
	case KeyEvent.VK_HOME:
	    dx = -1;
	    dy = -1;
	    break;
	case KeyEvent.VK_NUMPAD8:
	case KeyEvent.VK_UP:
	    dx = 0;
	    dy = -1;
	    break;
	case KeyEvent.VK_NUMPAD9:
	case KeyEvent.VK_PAGE_UP:
	    dx = 1;
	    dy = -1;
	    break;

	case KeyEvent.VK_NUMPAD4:
	case KeyEvent.VK_LEFT:
	    dx = -1;
	    dy = 0;
	    break;
	case KeyEvent.VK_NUMPAD6:
	case KeyEvent.VK_RIGHT:
	    dx = 1;
	    dy = 0;
	    break;

	case KeyEvent.VK_NUMPAD1:
	case KeyEvent.VK_END:
	    dx = -1;
	    dy = 1;
	    break;
	case KeyEvent.VK_NUMPAD2:
	case KeyEvent.VK_DOWN:
	    dx = 0;
	    dy = 1;
	    break;
	case KeyEvent.VK_NUMPAD3:
	case KeyEvent.VK_PAGE_DOWN:
	    dx = 1;
	    dy = 1;
	    break;

	case KeyEvent.VK_ESCAPE:
	    /* Exit game with escape. */
	    beasts.quit(BeastsWorld.QUIT_LEVEL);
	    return;

	    /* ignore all other keys. */
	default:
	    return;
	}

// 	System.err.println("Player.keyPressed(): " + ke
// 			   + " dx = " + dx
// 			   + " dy = " + dy);

	synchronized(beasts) {
	    WorldObject wo = beasts.getWO(x + dx, y + dy);

	    /* If we walked into a beast or an exploding HeavyBlock, we're
	     * dead. */
	    if(wo instanceof Beast
	       || (wo instanceof HeavyBlock
		   && ((HeavyBlock)wo).isExplosive())) {
		kill();
		beasts.update();
	    } else {
		/* If the square we're moving towards is empty, we're done;
		 * otherwise ask everything in that direction to move for
		 * us. If this fails, do nothing. */
		if(wo == null || wo.push(true, dx, dy)) {
		    int x0 = x, y0 = y;
		    beasts.move(this, x + dx, y + dy);

		    /* If we're pulling blocks and there's one behind us,
		     * move it too. */
		    if(ke.isShiftDown()
		       && pull
		       && (wo = beasts.getWO(x0 - dx, y0 - dy)) instanceof LightBlock) {
			beasts.move(wo, x0, y0);
		    }

		    beasts.update();
		}
	    }
	}
    }

    public void keyReleased(KeyEvent ke) {}
    public void keyTyped(KeyEvent ke) {}

    boolean push(boolean player, int dx, int dy) {
	WorldObject wo = beasts.getWO(x + dx, y + dy);

	/* Check to see if we got squashed, otherwise disallow the move. */
	if(wo instanceof Block) {
	    kill();
	    return true;
	} else {
	    return false;
	}
    }

    /**
     * <code>Beast</code> kills <code>Player</code>, either by running into
     * it or squashing it.
     */
    void kill() {
	beasts.removeWO(this);
	beasts.clear(x, y);

	if(beasts.training || lives-- > 0) {
	    /* Choose a random location, have another go. */
	    beasts.addPlayer();
	    beasts.updateStatus(false);
	} else {
	    /* Game's over. */
	    beasts.gameOver();
	}
    }

    Image getImage() {
	return image;
    }

    int getLives() {
	return lives;
    }

    public String toString() {
	return "Player[" + x + ", " + y + "]";
    }
}
