package beasts;

/**
 * Provides a few methods to do simple string formatting, and is otherwise
 * most identical to <code>StringBuffer</code>.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@unsw.edu.au">peteg@unsw.edu.au</A>)
 * @version $Id: StringFormatter.java,v 1.3 2002/02/19 10:33:46 peteg Exp $
 */
class StringFormatter {
    /* Justification. */
    static final int CENTRE = 0;
    static final int LEFT   = 1;
    static final int RIGHT  = 2;

    private StringBuffer sb;

    StringFormatter() {
	sb = new StringBuffer();
    }

    void append(char c) {
	sb.append(c);
    }

    void append(int i) {
	sb.append(i);
    }

    void append(String s) {
	sb.append(s);
    }

    void append(int i, int fw, int justification, char padding) {
	append(String.valueOf(i), fw, justification, padding);
    }

    /**
     * @arg fw The width of the field.
     */
    void append(String s, int fw, int justification, char padding) {
	switch(justification) {
	case CENTRE:
	    for(int i = (fw - s.length()) / 2; i > 0; i--) {
		append(padding);
	    }
	    append(s);
	    for(int i = (fw - s.length() + 1) / 2; i > 0; i--) {
		append(padding);
	    }
	    break;
	case LEFT:
	    append(s);
	    for(int i = fw - s.length(); i > 0; i--) {
		append(padding);
	    }
	    break;
	case RIGHT:
	    for(int i = fw - s.length(); i > 0; i--) {
		append(padding);
	    }
	    append(s);
	    break;
	default:
	    throw new RuntimeException("SF.append(): unknown justification: " + justification);
	}
    }

    void clear() {
	sb.setLength(0);
    }

    public String toString() {
	return sb.toString();
    }
}
