package beasts;

import java.awt.*;

/**
 * Super beasts. Can only be squashed against a <code>HeavyBlock</code>, and
 * can also be pregnant.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@unsw.edu.au">peteg@unsw.edu.au</A>)
 * @version $Id: SuperBeast.java,v 1.11 2002/02/19 10:33:46 peteg Exp $
 */
class SuperBeast extends Beast {
    static Image image;

    SuperBeast(BeastsWorld beasts, int x, int y) {
	super(beasts, x, y);
    }

    Image getImage() {
	return image;
    }

    int points() {
	return 5;
    }

    /**
     * Override this method to implement the "must squash against
     * <code>HeavyBlock</code>" policy.
     */
    boolean push(boolean player, int dx, int dy) {
	WorldObject wo = beasts.getWO(x + dx, y + dy);

	/* Check to see if we got squashed, otherwise disallow the move. */
	if(wo instanceof HeavyBlock) {
	    beasts.removeWO(this);
	    dead = true;
	    beasts.addScore(points());
	    return true;
	} else {
	    return false;
	}
    }

    public void move() {
	if(dead) {
	    /* Damn, we got squashed. No more turns for us. */
	    return;
	}

	/* Simplistic: move towards the player. */
	int dx = beasts.player.x - x,
	    dy = beasts.player.y - y;

	/* Squash the vector to {-1, 0, 1} ^ 2 */
	dx = (dx == 0) ? 0 : (dx < 0) ? -1 : 1;
	dy = (dy == 0) ? 0 : (dy < 0) ? -1 : 1;

	WorldObject wo = beasts.getWO(x + dx, y + dy);

	if(wo == null || wo instanceof Player) {
	    /* See if we caught the player. */
	    if(wo instanceof Player) {
// 		System.err.println("Beast caught player!");
		beasts.player.kill();
	    }

	    beasts.move(this, x + dx, y + dy);
	} else {
	    /* Choose a random direction. */
	    int[] dirs = new int[2 * 8];
	    int dirs_count = beasts.getValidDirections(x, y, dirs);

	    if(dirs_count > 0) {
		int dir = 2 * beasts.rnd(dirs_count);

// 		System.err.println("SuperBeast: random: dirs_count: "
// 				   + dirs_count + " dir: " + dir / 2);

		beasts.move(this, dirs[dir], dirs[dir + 1]);
	    }
	}

	beasts.update();
	beasts.add(this, beasts.BEAST_DELAY);
	return;
    }

    public String toString() {
	return "SuperBeast[" + x + ", " + y + "]";
    }
}
